class BaseException(Exception):
    def __init__(self, message=None):
        self.message = message

    def __str__(self):
        return f"{self.message}"


class UnsupportedArchiveFormat(BaseException):
    def __init__(self, message="Unsupported Archive Format."):
        super().__init__(message)


class WrongFileInfo(BaseException):
    def __init__(self, message="Either a URL or a file must be provided."):
        super().__init__(message)


class NumpyLoadError(BaseException):
    def __init__(self, message="Failed to load the .npy file."):
        super().__init__(message)


class NotSupportedFramework(BaseException):
    def __init__(self, message="Only .onnx and .tflite files are supported."):
        super().__init__(message)


class NotLoadableONNX(BaseException):
    def __init__(self, message="Can not load this onnx file."):
        super().__init__(message)


class NotLoadableTFLITE(BaseException):
    def __init__(self, message="Can not load this tflite file."):
        super().__init__(message)


class WrongDatsetFile(BaseException):
    def __init__(self, message="Dataset file is a not archive file neither '.npy' file.",):
        super().__init__(message)


import os
import tempfile
from pathlib import Path
import zipfile
import tarfile
import shutil

import py7zr
import rarfile
from loguru import logger

from .exceptions import UnsupportedArchiveFormat


def make_temp_dir()->str:
    temp_file_path = tempfile.mkdtemp()
    Path(temp_file_path).mkdir(exist_ok=True, parents=True)
    return temp_file_path


def compress_files(file_paths:str, zip_path:str):
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for file in file_paths:
            zipf.write(file, arcname=file.split('/')[-1])
    logger.info(f"Compress files: {file_paths}")


def extract_archive(archive_path:str, extract_path:str):
    if archive_path.endswith('.zip'):
        with zipfile.ZipFile(archive_path, 'r') as zip_ref:
            zip_ref.extractall(extract_path)
    elif archive_path.endswith(('.tar', '.tar.gz', '.tgz')):
        with tarfile.open(archive_path, 'r:*') as tar_ref:
            tar_ref.extractall(extract_path)
    elif archive_path.endswith('.7z'):
        with py7zr.SevenZipFile(archive_path, mode='r') as z:
            z.extractall(extract_path)
    elif archive_path.endswith('.rar'):
        with rarfile.RarFile(archive_path, 'r') as rar_ref:
            rar_ref.extractall(extract_path)
    else:
        raise UnsupportedArchiveFormat()


def delete_parent_directory(file_path:str):
    try:
        parent_directory = os.path.dirname(file_path)
        shutil.rmtree(parent_directory)
        logger.info(f"Directory {file_path} is deleted.")
    except FileNotFoundError:
        logger.info(f"Directory {file_path} does not exist.")
    except PermissionError:
        logger.info(f"Permission denied to delete {file_path}.")


import os
from pathlib import Path
from typing import Dict, Any

import numpy as np
from loguru import logger

from .tflite_inference import TFLITE
from .onnx_inference import ONNX
from .data_loader import NumpyDataLoader
from ..utils import make_temp_dir, compress_files, delete_parent_directory
from ..exceptions import NotSupportedFramework


class InferenceService:
    def __init__(self, model_file_path:str, dataset_file_path:str, num_threads:int=1):
        self.model_file_path = model_file_path
        self.dataset_file_path = dataset_file_path
        self.model_obj, self.inputs, self.outputs = self.set_model_obj(model_file_path, num_threads)
        self.data_loader = NumpyDataLoader(dataset_file_path, self.inputs)
        self.data_loader.load_datasets(dataset_file_path, self.inputs)
        self.result_save_path = make_temp_dir()

    def set_model_obj(self, model_file_path:str, num_threads:int):
        suffix = Path(model_file_path).suffix
        if suffix == ".tflite":
            model_obj = TFLITE(model_file_path, num_threads)
        elif suffix == ".onnx":
            model_obj = ONNX(model_file_path, num_threads)
        else:
            raise NotSupportedFramework()
        # TODO: check framework by model file inspector
        # model_info = validate_model_file(model_file_path)
        # if model_info[Enums.FRAMEWORK] == Enums.TFLITE:
        #     model_obj = TFLITE(model_file_path, kwargs["num_threads"])
        # elif model_info[Enums.FRAMEWORK] == Enums.ONNX:
        #     model_obj = ONNX(model_file_path)
        
        if len(model_obj.inputs) > 1:
            logger.info(f'{self.model_file_path} has {len(model_obj.inputs)} nodes for input layer')
        if len(model_obj.outputs) > 1:
            logger.info(f'{self.model_file_path} has {len(model_obj.outputs)} nodes for output layer')
        return model_obj, model_obj.inputs, model_obj.outputs

    def inference(self):
        try:
            return self.model_obj.inference(self.data_loader.npy)
        except Exception as e: # TODO make a exception
            raise e

    def postprocess(self, inference_results:Dict[Any, np.ndarray]):
        # save npy file for each layers result
        files_path = []
        result_file_path = os.path.join(self.result_save_path, "archive.zip")
        for k, v in inference_results.items():
            npy_file_path = os.path.join(self.result_save_path, f"{k}.npy")
            np.save(npy_file_path, inference_results[k])
            files_path.append(npy_file_path)
        # zip npy files
        compress_files(files_path, result_file_path)
        self.result_file_path = result_file_path
    
    def run(self):
        inference_results = self.inference()
        logger.info(f"Inference success: {self.model_file_path}")
        self.postprocess(inference_results)


if __name__ == "__main__":

    inf_service = InferenceService(
        model_file_path="/app/tests/people_detection.onnx",
        dataset_file_path="/app/tests/dataset_for_onnx.npy"
        )
    inf_service.run()